@@ -1,23 +1,28 @@
-/* Color setup for S_Lang screen library
-   Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2007, 2008, 2009 Free Software Foundation, Inc.
+/*
+   Color setup for S_Lang screen library
+
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    Written by:
-   Andrew Borodin <aborodin@vmail.ru>, 2009.
+   Andrew Borodin <aborodin@vmail.ru>, 2009
+   Egmont Koblinger <egmont@gmail.com>, 2010
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This file is part of the Midnight Commander.
 
-   This program is distributed in the hope that it will be useful,
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** \file color-slang.c
  *  \brief Source: S-Lang-specific color setup
@@ -36,8 +41,6 @@
 #include "color.h"              /* variables */
 #include "color-internal.h"
 
-#include "src/setup.h"          /* color_terminal_string */
-
 /*** global variables ****************************************************************************/
 
 /*** file scope macro definitions ****************************************************************/
@@ -61,18 +64,17 @@ has_colors (gboolean disable, gboolean force)
     {
         const char *terminal = getenv ("TERM");
         const size_t len = strlen (terminal);
+        char *cts = mc_global.tty.color_terminal_string;
 
-        char *cts = color_terminal_string;
-        char *s;
-        size_t i;
-
-        /* check color_terminal_string */
+        /* check mc_global.tty.color_terminal_string */
         while (*cts != '\0')
         {
+            char *s;
+            size_t i = 0;
+
             while (*cts == ' ' || *cts == '\t')
                 cts++;
             s = cts;
-            i = 0;
 
             while (*cts != '\0' && *cts != ',')
             {
@@ -141,7 +143,6 @@ tty_color_deinit_lib (void)
 void
 tty_color_try_alloc_pair_lib (tty_color_pair_t * mc_color_pair)
 {
-    const char *fg, *bg;
     if (mc_color_pair->ifg <= (int) SPEC_A_REVERSE)
     {
         switch (mc_color_pair->ifg)
@@ -168,9 +169,12 @@ tty_color_try_alloc_pair_lib (tty_color_pair_t * mc_color_pair)
     }
     else
     {
-        fg = (mc_color_pair->cfg) ? mc_color_pair->cfg : "default";
-        bg = (mc_color_pair->cbg) ? mc_color_pair->cbg : "default";
+        const char *fg, *bg;
+
+        fg = tty_color_get_name_by_index (mc_color_pair->ifg);
+        bg = tty_color_get_name_by_index (mc_color_pair->ibg);
         SLtt_set_color (mc_color_pair->pair_index, (char *) "", (char *) fg, (char *) bg);
+        SLtt_add_color_attribute (mc_color_pair->pair_index, mc_color_pair->attr);
     }
 }
 
@@ -202,3 +206,11 @@ tty_set_normal_attrs (void)
 }
 
 /* --------------------------------------------------------------------------------------------- */
+
+gboolean
+tty_use_256colors (void)
+{
+    return (SLtt_Use_Ansi_Colors && SLtt_tgetnum ((char *) "Co") == 256);
+}
+
+/* --------------------------------------------------------------------------------------------- */
