@@ -1,9 +1,8 @@
 /*
    Dialog box features module for the Midnight Commander
 
-   Copyright (C) 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2007, 2009, 2010, 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
 
@@ -51,6 +50,7 @@
 /* Color styles for normal and error dialogs */
 dlg_colors_t dialog_colors;
 dlg_colors_t alarm_colors;
+dlg_colors_t listbox_colors;
 
 /* Primitive way to check if the the current dialog is our dialog */
 /* This is needed by async routines like load_prompt */
@@ -493,7 +493,7 @@ dlg_key_event (WDialog * h, int d_key)
             dlg_one_down (h);
             return;
         }
-        else if (d_key == KEY_BTAB)
+        else if ((d_key & ~(KEY_M_SHIFT | KEY_M_CTRL)) == '\t')
         {
             dlg_one_up (h);
             return;
@@ -520,6 +520,7 @@ dlg_key_event (WDialog * h, int d_key)
     if (handled == MSG_NOT_HANDLED)
         handled = dlg_handle_key (h, d_key);
 
+    (void) handled;
     send_message (h, NULL, MSG_POST_KEY, d_key, NULL);
 }
 
@@ -528,7 +529,6 @@ dlg_key_event (WDialog * h, int d_key)
 static void
 frontend_dlg_run (WDialog * h)
 {
-    int d_key;
     Gpm_Event event;
 
     event.x = -1;
@@ -542,6 +542,8 @@ frontend_dlg_run (WDialog * h)
 
     while (h->state == DLG_ACTIVE)
     {
+        int d_key;
+
         if (mc_global.tty.winch_flag != 0)
             dialog_change_screen_size ();
 
@@ -698,7 +700,7 @@ dlg_set_position (WDialog * h, int y1, int x1, int y2, int x2)
 }
 
 /* --------------------------------------------------------------------------------------------- */
-/** this function sets only size, leaving positioning to automatic methods */
+/** Set dialog size and position */
 
 void
 dlg_set_size (WDialog * h, int lines, int cols)
@@ -706,14 +708,19 @@ dlg_set_size (WDialog * h, int lines, int cols)
     int x = WIDGET (h)->x;
     int y = WIDGET (h)->y;
 
-    if (h->flags & DLG_CENTER)
+    if ((h->flags & DLG_CENTER) != 0)
     {
         y = (LINES - lines) / 2;
         x = (COLS - cols) / 2;
     }
 
-    if ((h->flags & DLG_TRYUP) && (y > 3))
-        y -= 2;
+    if ((h->flags & DLG_TRYUP) != 0)
+    {
+        if (y > 3)
+            y -= 2;
+        else if (y == 3)
+            y = 2;
+    }
 
     dlg_set_position (h, y, x, y + lines, x + cols);
 }
@@ -778,8 +785,7 @@ dlg_create (gboolean modal, int y1, int x1, int lines, int cols,
 
     new_d->state = DLG_CONSTRUCT;
     new_d->modal = modal;
-    if (colors != NULL)
-        memmove (new_d->color, colors, sizeof (dlg_colors_t));
+    new_d->color = colors;
     new_d->help_ctx = help_ctx;
     new_d->flags = flags;
     new_d->data = NULL;
@@ -800,7 +806,7 @@ dlg_create (gboolean modal, int y1, int x1, int lines, int cols,
         g_free (t);
     }
 
-    /* unique name got event group for this dialog */
+    /* unique name of event group for this dialog */
     new_d->event_group = g_strdup_printf ("%s_%p", MCEVENT_GROUP_DIALOG, (void *) new_d);
 
     return new_d;
@@ -822,6 +828,12 @@ dlg_set_default_colors (void)
     alarm_colors[DLG_COLOR_HOT_NORMAL] = ERROR_HOT_NORMAL;
     alarm_colors[DLG_COLOR_HOT_FOCUS] = ERROR_HOT_FOCUS;
     alarm_colors[DLG_COLOR_TITLE] = ERROR_TITLE;
+
+    listbox_colors[DLG_COLOR_NORMAL] = PMENU_ENTRY_COLOR;
+    listbox_colors[DLG_COLOR_FOCUS] = PMENU_SELECTED_COLOR;
+    listbox_colors[DLG_COLOR_HOT_NORMAL] = PMENU_ENTRY_COLOR;
+    listbox_colors[DLG_COLOR_HOT_FOCUS] = PMENU_SELECTED_COLOR;
+    listbox_colors[DLG_COLOR_TITLE] = PMENU_TITLE_COLOR;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1192,11 +1204,13 @@ dlg_init (WDialog * h)
 
     h->state = DLG_ACTIVE;
 
-    /* Select the first widget that takes focus */
+    /* first send MSG_DRAW to dialog itself and all widgets... */
+    dlg_redraw (h);
+
+    /* ...then send MSG_FOCUS to select the first widget that can take focus */
     while (h->current != NULL && !dlg_focus (h))
         h->current = dlg_widget_next (h, h->current);
 
-    dlg_redraw (h);
 
     h->ret_value = 0;
 }
@@ -1262,8 +1276,7 @@ dlg_destroy (WDialog * h)
     /* if some widgets have history, save all history at one moment here */
     dlg_save_history (h);
     dlg_broadcast_msg (h, MSG_DESTROY);
-    g_list_foreach (h->widgets, (GFunc) g_free, NULL);
-    g_list_free (h->widgets);
+    g_list_free_full (h->widgets, g_free);
     mc_event_group_del (h->event_group);
     g_free (h->event_group);
     g_free (h->title);
