@@ -39,13 +39,17 @@ extern int option_return_does_auto_indent;
 extern int option_backspace_through_tabs;
 extern int option_fake_half_tabs;
 extern int option_persistent_selections;
+extern int option_drop_selection_on_copy;
 extern int option_cursor_beyond_eol;
+extern gboolean option_cursor_after_inserted_block;
 extern int option_line_state;
 extern int option_save_mode;
 extern int option_save_position;
 extern int option_syntax_highlighting;
 extern int option_group_undo;
 extern char *option_backup_ext;
+extern char *option_filesize_threshold;
+extern char *option_stop_format_chars;
 
 extern int edit_confirm_save;
 
@@ -62,11 +66,11 @@ extern int show_right_margin;
 void edit_stack_init (void);
 void edit_stack_free (void);
 
-gboolean edit_file (const vfs_path_t * file_vpath, int line);
+gboolean edit_file (const vfs_path_t * file_vpath, long line);
 gboolean edit_files (const GList * files);
 
 char *edit_get_file_name (const WEdit * edit);
-int edit_get_curs_col (const WEdit * edit);
+long edit_get_curs_col (const WEdit * edit);
 const char *edit_get_syntax_type (const WEdit * edit);
 
 /*** inline functions ****************************************************************************/
