@@ -1,8 +1,8 @@
 /*
    lib/vfs - test vfs_split() functionality
 
-   Copyright (C) 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 2011-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Slava Zanko <slavazanko@gmail.com>, 2011, 2013
